<?php

namespace Teknisa\MAN\Controllers;

use Teknisa\MAN\Services\GeralFilialService;
use Teknisa\MAN\Factories\GeralFilialRepository;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;

class GeralFilial extends CrudWithParameterBag{

	protected $dataSourceName = 'geralfilial';
    protected $dataSourceNameGetTreeFilial = 'gettreefilial';
    protected $parameterBagColumns = array('NRORG', 'CDFILIAL', 'NMFILIAL');
	protected $geralFilialService;
    protected $geralFilialRepository;
    protected $entityManager;


	public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, GeralFilialService $geralFilialService, GeralFilialRepository $geralFilialRepository, EntityManager $entityManager) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->geralFilialService = $geralFilialService;
        $this->geralFilialRepository = $geralFilialRepository;
        $this->entityManager = $entityManager;
    }

    public function GetTreeFilial(DTO\Request\Filter $request, DTO\Response $response) {
    // 	$tablesTreatment = [];
    // 	$tablesClause = [];

    //     $filterCriteria = $request->getFilterCriteria();
    //     foreach ($filterCriteria->getConditions() as $condition) {
    //         $columnName = $condition['columnName'];
    //         if($columnName == 'Research Treatment') {
    //         	array_push($tablesTreatment, $condition['value']);
    //         }else {
				// array_push($tablesClause, $condition);            	
    //         }
    //     }
    //     $dataSet = $this->geralFilialService->mountsQueryFilial($tablesClause, $tablesTreatment);
    //     $response->addDataSet(new DataSet('getTreeFilial', $dataSet));
        $this->dataSourceName = $this->dataSourceNameGetTreeFilial;
        self::find($request, $response);
    }

    public function GetAlreadyAdd(Request\DataSet $request, Response $response){
        $dataSet = array();
        foreach ($request->getDataSet()->getRows() as $row) {
            $object = array();
            $object['TYPE'] = "SELECT";
            $object['ROW'] = $row;
            $comp = $row['TOSAVE'];
            $exist = array();
            foreach ($comp as $goTest) {
                $row['TOSAVE'] = $goTest;
                $query = $this->geralFilialService->mountQuery($object);
                $resp = $this->geralFilialRepository->executeQuery($query);
                if (!empty($resp)){
                    array_push($exist, $resp[0]);
                    $row['CANTSAVE'] = $goTest;
                }        
            }
        }
        $row['TOSAVE'] = $comp;
        if (!empty($exist)){
            array_push($dataSet, $row, true, $exist);
        }
        else {
            array_push($dataSet, $row, false);   
        }
        $response->addDataSet(new DataSet('getAlreadyAdd', $dataSet));
    }

    public function SaveGeralFilial(Request\DataSet $request, Response $response){
        $dataSet = array();
        try{
            $this->entityManager->getConnection()->beginTransaction();   //--- Começa uma transação sem auto commit  no banco
            foreach ($request->getDataSet()->getRows() as $row) {
                $pk = $this->geralFilialRepository->GetNextIdPk($row['TABLE'], $row['PKTABLE']);
                $row['FIELDSTOSAVE'][$pk[0]] = $pk[1]; 
                $queryToSave = $this->geralFilialService->mountQuery($row);
                $this->geralFilialRepository->Save($queryToSave);
            }
            $this->entityManager->getConnection()->commit(); //---- Grava a transação de fato no banco (commit)
            array_push($dataSet, true);
            $response->addDataSet(new DataSet('saveGeralFilial', $dataSet));
            $response->addNotification(new Notification("Salvo com sucesso!", Notification::TYPE_SUCCESS));
        }catch(\Exception $e){
            $message = $e->getMessage();
            $this->entityManager->getConnection()->rollBack(); //--- Descarta a transação
            array_push($dataSet, false, $message);
            $response->addDataSet(new DataSet('saveGeralFilial', $dataSet));
        }
    }

    public function GetGridTree(DTO\Request\Filter $request, DTO\Response $response){
        $filterCriteria = $request->getFilterCriteria()->getConditions();
        
        $object = array();
        $whereQuery = array(); //colunas por quem eu filtro. ex= :NRORG
        $tableName = ""; //nome da tabela de relacionamento que busca os dados
        $selectQuery = array(); //colunas que estarao no select
        $researchTreatment = array();   //para telas com 3 níveis no grid

        foreach ($filterCriteria as $condition) {
            if ($condition['columnName'] != 'TABLE' && $condition['columnName'] != 'SELECTFIELDS'){
                $obj = array();
                $obj['NAME'] = $condition['columnName'];
                $obj['VALUE'] = $condition['value'];
                array_push($whereQuery, $obj);
            }
            elseif ($condition['columnName'] == 'TABLE') {
                $table = explode(" " , $condition['value']); 
                $object['TABLE'] = $table[0];
                $object['PKTABLE'] = $table[1];
            }
            elseif ($condition['columnName'] == 'SELECTFIELDS'){
                foreach ($condition['value'] as $value) {
                    $columnName = explode(" ", $value);
                    if (count($columnName) > 1){
                        foreach ($columnName as $columnOrTable) {
                            array_push($researchTreatment, $columnOrTable);
                        }
                    }
                    else {
                        array_push($selectQuery, $columnName[0]);
                    }
                }
            }    
        }
        $object['TYPE'] = "FILTER";
        $object['WHERE'] = $whereQuery;
        $object['ADDSELECT'] = $selectQuery;
        $object['TREATMENT'] = $researchTreatment;
        
        $queryFilter = $this->geralFilialService->mountQuery($object);
        $data = $this->geralFilialRepository->executeQuery($queryFilter);
        $response->addDataSet(new DataSet('getGridTree', $data));
    }

    public function DeleteRelationship(Request\DataSet $request, Response $response){
        $rows = $request->getDataSet()->getRows();
        $dataSet = array();
        
        foreach ($rows as $object) {
            $object['TYPE'] = "DELETE";
            $queryToDelete = $this->geralFilialService->mountQuery($object);
        }

        $result = $this->geralFilialRepository->Execute($queryToDelete);
        $response->addDataSet(new DataSet('deleteRelationship', $result));
    }
}